#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome check --run=ID [--cpu=INT] [-v] [--vv]

options:
    --run=ID    Pathname to the comparison workspace.
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
    -v     Verbose.
    --vv    Very verbose.
"""

import csv
import docopt
import os
import time

from padmet.utils.connection import gbk_to_faa, pgdb_to_padmet, sbmlGenerator

from aucome.utils import parse_config_file
from aucome.reconstruction import create_padmet_from_pgdb, create_sbml

from Bio import SeqIO
from multiprocessing import Pool


def command_help():
    print(docopt.docopt(__doc__))


def check_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    verbose = args['-v']
    veryverbose = args['--vv']

    if args["--cpu"]:
        nb_cpu_to_use = int(args["--cpu"])
    else:
        nb_cpu_to_use = 1

    if veryverbose and not verbose:
        verbose = veryverbose

    run_check(run_id, nb_cpu_to_use, verbose, veryverbose)


def run_check(run_id, nb_cpu_to_use, verbose, veryverbose):
    if verbose:
        print('--- Running check step ---')
    start_time = time.time()

    config_data = parse_config_file(run_id)

    padmet_from_annotation_path = config_data['padmet_from_annotation_path']
    study_from_annot_prefix = config_data['study_from_annot_prefix']
    sbml_from_annotation_path = config_data['sbml_from_annotation_path']
    database_path = config_data['database_path']
    pgdb_from_annotation_path = config_data['pgdb_from_annotation_path']
    studied_organisms_path = config_data['studied_organisms_path']
    analysis_group_file_path = config_data['analysis_group_file_path']

    #create dict for ortho data
    all_study_name = set(next(os.walk(studied_organisms_path))[1])
    all_study_pgdb = dict([(study_name, "{0}/{1}".format(pgdb_from_annotation_path, study_name))
                          if os.path.isdir("{0}/{1}".format(pgdb_from_annotation_path, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])
    all_study_gbk = dict([(study_name, "{0}/{1}/{1}.gbk".format(studied_organisms_path, study_name))
                          if os.path.isfile("{0}/{1}/{1}.gbk".format(studied_organisms_path, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])

    # Update group file in analysis
    if not os.path.exists(analysis_group_file_path):
        with open(analysis_group_file_path, 'w') as group_file:
            group_writer = csv.writer(group_file, delimiter='\t')
            group_writer.writerow(['all', *all_study_name])
    else:
        groups_data = []
        with open(analysis_group_file_path, 'r') as group_file:
            group_reader = csv.reader(group_file, delimiter='\t')
            for row in group_reader:
                groups = [org_name for org_name in row[1:] if org_name]
                groups_data.append((row[0], groups))

        # Check if 'all' row matches species in study_organisms.
        if sorted(groups_data[0][1]) != sorted(all_study_name):
            with open(analysis_group_file_path, 'w') as group_file:
                group_writer = csv.writer(group_file, delimiter='\t')
                group_writer.writerow(['all', *all_study_name])
                for group in groups_data:
                    if group[0] != 'all':
                        group_writer.writerow([group[0], *group[1]])

    aucome_pool = Pool(nb_cpu_to_use)

    if verbose:
        print('Checking genbank file.')
    study_faa_data = []
    for study_name in all_study_name:
        faa_path = "{0}/{1}/{1}.faa".format(studied_organisms_path, study_name)
        tmp_faa_data = {'study_name': study_name, 'faa_path': faa_path, 'gbk_file': all_study_gbk[study_name],
                        'studied_organisms_path': studied_organisms_path,
                        'verbose': verbose}
        study_faa_data.append(tmp_faa_data)
    aucome_pool.map(check_create_faa, study_faa_data)

    #k = folder_name in studied_org_path, v = path to faa in this folder, faa name should be folder_name.faa
    all_study_faa = dict([(study_name, "{0}/{1}/{1}.faa".format(studied_organisms_path, study_name))
                          if os.path.isfile("{0}/{1}/{1}.faa".format(studied_organisms_path, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])

    study_padmet_data = []
    for study_name in all_study_name:
        padmet_file = "{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name)
        pgdb_folder = all_study_pgdb[study_name]
        tmp_padmet_data = {'study_name': study_name, 'pgdb_folder': pgdb_folder, 'verbose': verbose, 
                           'padmet_file': padmet_file, 'database_path': database_path, 'veryverbose': veryverbose}
        study_padmet_data.append(tmp_padmet_data)
    aucome_pool.map(create_padmet_from_pgdb, study_padmet_data)

    all_study_padmet = dict([(study_name, "{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name))
                          if os.path.isfile("{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])

    study_sbml_data = []
    for study_name in all_study_padmet:
        sbml_file = "{0}/{1}{2}.sbml".format(sbml_from_annotation_path, study_from_annot_prefix, study_name)
        padmet_file = all_study_padmet[study_name]
        tmp_sbml_data = {'sbml_file': sbml_file, 'padmet_file': padmet_file,
                         'study_name': study_name, 'verbose': verbose, 'veryverbose': veryverbose}
        study_sbml_data.append(tmp_sbml_data)
    aucome_pool.map(create_sbml, study_sbml_data)

    #sbml of study are obtained from annotation, they should be in sbml_from_annotation_path
    #k = study_name (== folder_name in studied_org_path or obtained from sbml name), v = path to sbml, sbml_study_prefi+study_name+.sbml
    all_study_sbml = dict([(study_name, "{0}/{1}{2}.sbml".format(sbml_from_annotation_path, study_from_annot_prefix, study_name))
                           if os.path.isfile("{0}/{1}{2}.sbml".format(sbml_from_annotation_path, study_from_annot_prefix, study_name))
                           else (study_name, '')
                           for study_name in all_study_name])
    #PGDB, padmet, sbml
    all_study_pgdb = dict([(study_name, "{0}/{1}".format(pgdb_from_annotation_path, study_name))
                          if os.path.isdir("{0}/{1}".format(pgdb_from_annotation_path, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])

    if verbose:
        print("Input summary:")
        print("* %s Studied organims:" %(len(all_study_name)))
        for study_name in all_study_name:
            print("%s:" %study_name)
            if all_study_gbk[study_name]:
                print("\tGBK: OK")
            else:
                print("\t[WARNING] No GBK found, should be in {1}/{0}/{0}.gbk".format(study_name, studied_organisms_path))
            if all_study_faa[study_name]:
                print("\tFAA: OK")
            else:
                print("\t[WARNING] No FAA found, should be in {1}/{0}/{0}.faa".format(study_name, studied_organisms_path))
            if all_study_pgdb[study_name]:
                print("\tPGDB: OK")
            else:
                print("\t[WARNING] No PGDB found, should be in {1}/{0}".format(study_name, pgdb_from_annotation_path))
            if all_study_padmet[study_name]:
                print("\tPADMET: OK")
            else:
                print("\t[WARNING] No PADMET found, should be in {1}/{2}{0}.padmet".format(study_name, padmet_from_annotation_path, study_from_annot_prefix))
            if all_study_sbml[study_name]:
                print("\tSBML: OK")
            else:
                print("\t[WARNING] No SBML found, should be in {1}/{2}{0}.sbml".format(study_name, sbml_from_annotation_path, study_from_annot_prefix))
        
    aucome_pool.close()
    aucome_pool.join()

    end_time = (time.time() - start_time)
    integer_part, decimal_part = str(end_time).split('.')
    check_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- check step done in: %ss ---" %check_time)


def check_create_faa(tmp_faa_data):
    study_name = tmp_faa_data['study_name']
    faa_path = tmp_faa_data['faa_path']
    gbk_file = tmp_faa_data['gbk_file']
    verbose = tmp_faa_data['verbose']
    studied_organisms_path = tmp_faa_data['studied_organisms_path']
    checking_genbank(study_name, studied_organisms_path, verbose)

    #create Faa from gbk if no faa found
    if not os.path.isfile(faa_path) and gbk_file:
        if verbose:
            print("Creating faa from gbk for %s" %study_name)
        gbk_to_faa.gbk_to_faa(gbk_file=gbk_file, output=faa_path, verbose=verbose)


def checking_genbank(genbank_file_name, studied_organisms_path, verbose):
    """
    Check if there is a special character in the genbank file.
    If yes exit and print an error.

    Check gene ID in genbank to find too long gene ID or invalid character in gene ID.
    """
    invalid_characters = ['-', '|', '/', '(', ')', '\'', '=', '#', '*',
                '.', ':', '!', '+', '[', ']', ',', " "]
    if any(char in invalid_characters for char in genbank_file_name):
        print('Error in genbank file name: ' + genbank_file_name)
        print('Rename the file without:',genbank_file_name)

    # Path to the genbank file.
    genbank_path = studied_organisms_path + '/' + genbank_file_name + '/' + genbank_file_name + '.gbk'

    fix_dot_protein_seq = None
    invalid_gene_ids = []
    too_long_ids = []
    for record in SeqIO.parse(genbank_path, 'genbank'):
        for feature in record.features:
            if 'locus_tag' in feature.qualifiers:
                locus_tag = feature.qualifiers['locus_tag'][0]
                if any(char in invalid_characters for char in locus_tag):
                    if verbose:
                        invalid_gene_ids.append(feature)
                if len(locus_tag) >= 40:
                        too_long_ids.append(feature)
            if 'translation' in feature.qualifiers:
                if '.' in feature.qualifiers['translation'][0]:
                    fix_dot_protein_seq = True

    if len(invalid_gene_ids) > 0:
        print('Error of gene id in genbank ' + genbank_file_name + ', ' + str(len(invalid_gene_ids)) + ' genes have an invalid characters present: ' + ' '.join(invalid_characters) + '.')
    if len(too_long_ids) > 0:
        print('Error of gene id in genbank ' + genbank_file_name + ', ' + str(len(too_long_ids)) + ' genes have a gene id too long.')

    if len(invalid_gene_ids) > 0 or len(too_long_ids) > 0:
        print('Gene ID in ' + genbank_file_name + ' must be renamed.')
        fix_name = True
    else:
        fix_name = False

    if fix_dot_protein_seq:
        print('Dot in a protein sequence, Orthofinder will not work for this sequence. Dot will be deleted.')

    if fix_name or fix_dot_protein_seq:
        fix_genbank_file(genbank_file_name, fix_name, fix_dot_protein_seq, studied_organisms_path, verbose)


def adapt_gene_id(gene_id, longest_gene_number_length):
    """
    Input: a gene ID like g_1 and the longest_gene_number_length (5 if you have a gene like g_12356).
    Return a new gene ID with a completion of 0 like: g_00001.
    """
    gene_prefix = '_'.join(gene_id.split('_')[:-1])
    gene_number = gene_id.split('_')[-1]

    new_gene_id = gene_prefix + '_' + gene_number.zfill(longest_gene_number_length)

    return new_gene_id


def fix_genbank_file(genbank_file_name, fix_name, fix_dot_protein_seq, studied_organisms_path, verbose):
    # Path to the genbank file.
    genbank_path = studied_organisms_path + '/' + genbank_file_name + '/' + genbank_file_name + '.gbk'
    genbank_path_renamed = studied_organisms_path + '/' + genbank_file_name + '/' + genbank_file_name + '_original.gbk'

    if os.path.exists(genbank_path_renamed):
        print(genbank_file_name + ': Renaming has already been made on the data.')
        return

    # Create records that will be modified according to the issue: location or gene id.
    new_records = [record for record in SeqIO.parse(genbank_path, 'genbank')]

    # Use either the genbank accession or the genus + species as a prefix for new gene ID.
    try:
        new_prefix = new_records[0].annotations['accessions'][0] + '_' + str(new_records[0].annotations['sequence_version'])
    except:
        new_prefix = new_records[0].annotations['organism'].split(' ')[0][0] + '_' + new_records[0].annotations['organism'].split(' ')[1]

    if fix_name or fix_dot_protein_seq:
        # Dictionary wtih gene id as key and renamed id as value.
        feature_id_mappings = {}

        number_genes_genbanks = len([feature for record in SeqIO.parse(genbank_path, 'genbank') for feature in record.features if feature.type == 'gene'])
        gene_number = 1
        # Renamed ID: genbank file name + '_' + gene_position_number.
        # Max ID len is 39 for Pathway-Tools.
        for record in new_records:
            for feature in record.features:
                if 'locus_tag' in feature.qualifiers:
                    if fix_name:
                        feature_id = feature.qualifiers['locus_tag'][0]
                        if feature_id not in feature_id_mappings:
                            new_gene_id = new_prefix + '_' + str(gene_number)
                            new_feature_id = adapt_gene_id(new_gene_id, len(str(number_genes_genbanks)))
                            feature_id_mappings[feature_id] = new_feature_id
                            feature.qualifiers['locus_tag'][0] = new_feature_id
                            feature.qualifiers['old_locus_tag'] = feature_id
                            gene_number += 1
                        else:
                            feature.qualifiers['locus_tag'][0] = feature_id_mappings[feature_id]
                            feature.qualifiers['old_locus_tag'] = feature_id
                    if fix_dot_protein_seq:
                        if 'translation' in feature.qualifiers:
                            feature.qualifiers['translation'] = feature.qualifiers['translation'][0].replace('.', '')

        if fix_name:
            # Create a TSV mapping file with original and renamed ids.
            mapping_dic_path = studied_organisms_path + '/' + genbank_file_name + '/' + genbank_file_name + '_dict.csv'
            with open(mapping_dic_path, 'w') as csv_file:
                writer = csv.writer(csv_file, delimiter='\t')
                writer.writerow(["original_gene_id", "renamed_gene_id"])
                for key, value in list(feature_id_mappings.items()):
                    writer.writerow([key, value])

    # Create genbank with renamed id.
    new_genbank_path = studied_organisms_path + '/' + genbank_file_name + '/' + genbank_file_name + '_tmp.gbk'
    SeqIO.write(new_records, new_genbank_path, 'genbank')

    # Save original genbank.
    os.rename(genbank_path, genbank_path_renamed)

    # Rename renamed genbank to genbank used by the script.
    os.rename(new_genbank_path, genbank_path)

    if verbose:
        print(genbank_file_name + ' ids have been renamed.')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
     aucome orthology --run=ID [--sequence_search_prg=STR] [--cpu=INT] [-v] [--vv] [--filtering] [--threshold=FLOAT] [--union] [--intersection]
     
options:
     --run=ID    Pathname to the comparison workspace.
     --sequence_search_prg=STR    Sequence search program for Orthofinder [Default: diamond].
         Options: blast, mmseqs, blast_gz, diamond, and diamond_ultra_sens.
     --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
     -v     Verbose.
     --vv    Very verbose.
     --filtering     Use a filter to limit propagation, by default it is 0.05, if you want to modify the value use --threshold.
     --threshold=FLOAT     Threshold of the filter to limit propagation to use with the --filtering argument.
    --union          Use the union filter between five threshold values [0.01, 0.05, 0.1, 0.15, 0.2] to limit propagation, to use with the --filtering argument.
    --intersection   Use the intersection filter between five threshold values [0.01, 0.05, 0.1, 0.15, 0.2] to limit propagation, to use with the --filtering argument.
"""

import csv
import docopt
import os
import subprocess
import shutil
import sys
import time

from padmet.classes import PadmetSpec
from padmet.utils.exploration import convert_sbml_db
from padmet.utils.connection import extract_orthofinder
from padmet.utils.connection import sbml_to_padmet, sbmlGenerator

from aucome.utils import parse_config_file
from multiprocessing import Pool


def command_help():
    print(docopt.docopt(__doc__))


def orthology_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    sequence_search_prg = args['--sequence_search_prg']
    cpu = args['--cpu']
    verbose = args['-v']
    veryverbose = args['--vv']
    filtering = args['--filtering']
    threshold = args['--threshold']
    union = args['--union']
    intersection = args['--intersection']
    filtering_threshold_list = []
    
    if filtering:
        if threshold:
            try:
                filtering_threshold_list.append(float(threshold))
            except:
                sys.exit('filtering_threshold value must be a float between 0 and 1.')       
        else:
            filtering_threshold_list.append(0.05)      
        if union or intersection:
            filtering_threshold_list = [0.01, 0.05, 0.1, 0.15, 0.2]
    else:
        if threshold:
            sys.exit('--threshold must be used with --filtering.')
        if union:
            sys.exit('--union must be used with --filtering.')
        if intersection:
            sys.exit('--intersection must be used with --filtering.')
        
    if cpu:
        nb_cpu_to_use = int(cpu)
    else:
        nb_cpu_to_use = 1

    if veryverbose and not verbose:
        verbose = veryverbose

    run_orthology(run_id, sequence_search_prg, nb_cpu_to_use, filtering_threshold_list, union, intersection, verbose, veryverbose)


def run_orthology(run_id, sequence_search_prg, nb_cpu_to_use, filtering_threshold_list, union, intersection, verbose, veryverbose=None):
    print('--- Running orthology step ---')
    orthology_start_time = time.time()
    aucome_pool = Pool(nb_cpu_to_use)
    config_data = parse_config_file(run_id)

    orthofinder_wd_path = config_data['orthofinder_wd_path']
    orthofinder_bin_path = config_data['orthofinder_bin_path']
    orthofinder_sbml_path = config_data['orthofinder_sbml_path']
    orthofinder_padmet_path = config_data['orthofinder_padmet_path']
    orthofinder_filtered_path = config_data['orthofinder_filtered_path']
    studied_organisms_path = config_data['studied_organisms_path']
    padmet_from_annotation_path = config_data['padmet_from_annotation_path']
    database_path = config_data['database_path']

    all_study_name = set(next(os.walk(studied_organisms_path))[1])
    
    all_study_faa = dict([(study_name, "{0}/{1}/{1}.faa".format(studied_organisms_path, study_name))
                          if os.path.isfile("{0}/{1}/{1}.faa".format(studied_organisms_path, study_name))
                          else (study_name, '')
                          for study_name in all_study_name])

    #check if Orthofinder already run, if yes, get the last workdir.
    try:
    #    if orthogroups:
    #        orthodata_path = max(["%s/%s" %(x[0], 'Orthogroups/Orthogroups.tsv') for x in os.walk(orthofinder_wd_path) if 'Orthogroups' in x[1]])
    #    else:
        orthodata_path = max(["%s/%s" %(x[0], 'Orthologues') for x in os.walk(orthofinder_wd_path) if 'Orthologues' in x[1]])
    except ValueError:
        if verbose:
            print("Unable to find file Orthogroups.tsv in {0}, need to run Orthofinder...".format(orthofinder_wd_path))
        orthodata_path = None

    # If there is already an orthofinder result, check if all the species are in it.
    if orthodata_path:
        wd_orthodata_path = max(["%s/%s" %(x[0], 'WorkingDirectory') for x in os.walk(orthofinder_wd_path) if 'WorkingDirectory' in x[1]])

        input_fasta = [fasta_name for fasta_name in all_study_faa]
        already_analysed_fasta = [fasta_name.replace('.faa', '') for fasta_name in os.listdir(orthofinder_wd_path) if fasta_name != 'OrthoFinder']

        # If there is missing species, rerun OrthoFinder to add the missing species.
        if len(already_analysed_fasta) != len(input_fasta):
            fasta_to_adds = set(input_fasta) - set(already_analysed_fasta)
            tmp_folder = orthofinder_wd_path + '/tmp/'
            os.mkdir(tmp_folder)

            print('There is missing species in orthofinder: ' + ','.join(list(fasta_to_adds)))
            print('Rerun OrthoFinder on them using the old results from '+ orthofinder_wd_path)

            for name, faa_path in list(all_study_faa.items()):
                if name in fasta_to_adds:
                    if not os.path.isfile("{0}/{1}.faa".format(tmp_folder, name)):
                        if verbose:
                            print("Copying {0}'s faa to {1}".format(name, tmp_folder))
                    if faa_path != "" and os.path.isfile(faa_path):
                        shutil.copyfile(faa_path, tmp_folder + '/' + name + '.faa')
                    else:
                        sys.exit("Missing fasta for " + name + ", use 'aucome check' to input fasta from genbank.")

            orthofinder_result_path = orthofinder_wd_path + '/OrthoFinder/'
            start_time = time.time()
            cmds = [orthofinder_bin_path, "-b", wd_orthodata_path, "-f", tmp_folder,
                    "-t", str(nb_cpu_to_use), "-S", sequence_search_prg]
                       
            subprocess.call(cmds)
            end_time = (time.time() - start_time)
            integer_part, decimal_part = str(end_time).split('.')
            end_time = ".".join([integer_part, decimal_part[:3]])
            if verbose:
                print("Orthofinder done in: %ss" %end_time)

            for fasta_name in os.listdir(tmp_folder):
                shutil.copyfile(tmp_folder + '/' + fasta_name, orthofinder_wd_path + '/' + fasta_name)

            shutil.rmtree(tmp_folder)

            # Replace the old OrthoFinder results folder with the new one.
            new_orthofidner_path = max(["%s/%s" %(x[0], 'OrthoFinder') for x in os.walk(wd_orthodata_path) if 'OrthoFinder' in x[1]])

            orthofinder_tmp = orthofinder_wd_path + '/OrthoFinder_tmp'

            shutil.copytree(new_orthofidner_path, orthofinder_tmp)
            shutil.rmtree(orthofinder_result_path)
            shutil.copytree(orthofinder_tmp, orthofinder_result_path)
            shutil.rmtree(orthofinder_tmp)

            #if orthogroups:
            #    orthodata_path = max(["%s/%s" %(x[0], 'Orthogroups/Orthogroups.tsv') for x in os.walk(orthofinder_wd_path) if 'Orthogroups' in x[1]])
            #else:
            orthodata_path = max(["%s/%s" %(x[0], 'Orthologues') for x in os.walk(orthofinder_wd_path) if 'Orthologues' in x[1]])

            # Clean sbml/padmet/padmet filtered to recreate them with the new data.
            for sbml_folder in os.listdir(orthofinder_sbml_path):
                shutil.rmtree(orthofinder_sbml_path + '/' + sbml_folder)

            for padmet_folder in os.listdir(orthofinder_padmet_path):
                os.remove(orthofinder_padmet_path + '/' + padmet_folder)

            for filtered_padmet_folder in os.listdir(orthofinder_filtered_path):
                os.remove(orthofinder_filtered_path + '/' + filtered_padmet_folder)

    # If there is no OrthoFinder results folder run OrthoFinder on all the fasta.
    elif not orthodata_path:
        for name, faa_path in list(all_study_faa.items()):
            if not os.path.isfile("{0}/{1}.faa".format(orthofinder_wd_path, name)):
                if verbose:
                    print("Copying {0}'s faa to {1}".format(name, orthofinder_wd_path))
            if faa_path != "" and os.path.isfile(faa_path):
                shutil.copyfile(faa_path, orthofinder_wd_path + '/' + name + '.faa')
            else:
                sys.exit("Missing fasta for " + name + ", use 'aucome check' to input fasta from genbank.")
                
        if verbose:
            print("Running Orthofinder on %s cpu" %nb_cpu_to_use)

        start_time = time.time()
        cmds = [orthofinder_bin_path, "-f", orthofinder_wd_path, "-t", str(nb_cpu_to_use), "-S", sequence_search_prg]
        subprocess.call(cmds)
        end_time = (time.time() - start_time)
        integer_part, decimal_part = str(end_time).split('.')
        end_time = ".".join([integer_part, decimal_part[:3]])
        if verbose:
            print("Orthofinder done in: %ss" %end_time)

        orthofinder_result_path = orthofinder_wd_path + '/OrthoFinder/'
        if os.path.exists(orthofinder_result_path):
            orthologues_result_path = orthofinder_result_path + os.listdir(orthofinder_result_path)[0] + '/Orthologues'
            if os.path.exists(orthologues_result_path):
                orthodata_path = max(["%s/%s" %(x[0], 'Orthologues') for x in os.walk(orthofinder_wd_path) if 'Orthologues' in x[1]])
            else:
                sys.exit('There was an error with OrthoFinder, there is no results in ' + orthologues_result_path)
        else:
            sys.exit('Missing OrthoFinder folder in ' + orthofinder_wd_path)

    if verbose:
        print("Parsing Orthofinder output %s" %orthodata_path)

    if verbose:
        print("Start sbml creation...")
    all_dict_data = []
    for study_name in all_study_name:
        output_sbml = os.path.join(orthofinder_sbml_path, study_name)
        if os.path.exists(output_sbml):
            print(output_sbml + " already exists, delete it if you want to relaunch ortholog creation.")
        else:
            dict_data = {'sbml': run_id, 'orthodata_path': orthodata_path,
                         'study_name': study_name, 'verbose': verbose,
                         'veryverbose': veryverbose, 'output': output_sbml}
            all_dict_data.append(dict_data)

    start_time = time.time()
    aucome_pool.map(orthogroup_to_sbml, all_dict_data)
    end_time = (time.time() - start_time)
    integer_part, decimal_part = str(end_time).split('.')
    end_time = ".".join([integer_part, decimal_part[:3]])
    if verbose:
        print("Orthofinder output parsed in: %ss" %end_time)
    """
    #check database, mapping to metacyc ???
    data_convert_sbml_db = []
    for dict_data in all_dict_data:
        tmp_dict_data = {'sbml': orthology_based_path + '/' + study_name, 
                         'mnx_rxn_path': mnx_rxn_path, 'mnx_cpd_path': mnx_cpd_path, 'verbose': verbose}
        data_convert_sbml_db.append(tmp_dict_data)
        
    aucome_pool.map(_convert_sbml_db, data_convert_sbml_db)
    """

    if verbose:
        if len(filtering_threshold_list)>0 :
            print("Start padmet creation and filtering...")
        else:
            print("Start padmet creation...")

    multiprocessing_datas = []
    update_padmet_datas = []
    for sbml in os.listdir(orthofinder_sbml_path):
        input_pwt_padmet = padmet_from_annotation_path + '/output_pathwaytools_' + sbml + '.padmet'
        output_padmet = orthofinder_padmet_path + '/' + sbml + '.padmet'
        if os.path.exists(output_padmet):
            print(output_padmet + " already exists, delete it if you want to relaunch ortholog creation.")
        else:
            multiprocessing_datas.append([sbml, orthofinder_sbml_path, input_pwt_padmet,
                            database_path, output_padmet, orthodata_path,
                            orthofinder_filtered_path, filtering_threshold_list, verbose, veryverbose])
            update_padmet_datas.append([sbml, orthodata_path, output_padmet, verbose])

    start_time = time.time()
    aucome_pool.starmap(orthology_to_padmet, multiprocessing_datas)

    # Add the orthologs to the padmets.
    if verbose:
        print("Updating padmets...")
    aucome_pool.starmap(addOrthologyInPadmet, update_padmet_datas)

    if len(filtering_threshold_list)>0:
        filter_propagation(orthofinder_padmet_path, orthofinder_filtered_path, aucome_pool, filtering_threshold_list, union, intersection, verbose)
   
    end_time = (time.time() - start_time)
    integer_part, decimal_part = str(end_time).split('.')
    end_time = ".".join([integer_part, decimal_part[:3]])
    if verbose:
        print("Padmet created in: %ss" %end_time)

    aucome_pool.close()
    aucome_pool.join()

    orthology_end_time = (time.time() - orthology_start_time)
    integer_part, decimal_part = str(orthology_end_time).split('.')
    orthology_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- orthology step done in: %ss ---" %orthology_time)


def _convert_sbml_db(data_convert_sbml_db):
    
    sbml_file = data_convert_sbml_db['sbml']
    verbose = data_convert_sbml_db['verbose']
    mnx_rxn_path = data_convert_sbml_db['mnx_rxn_path']
    mnx_cpd_path = data_convert_sbml_db['mnx_cpd_path']

    if os.path.isfile(sbml_file):
        dict_file = "{0}_dict.tsv".format(os.path.splitext(sbml_file)[0])
        if not os.path.exists(dict_file):
            db_ref = convert_sbml_db.check_sbml_db(sbml_file, "reaction", mnx_reac_file=mnx_rxn_path, verbose=True)[0]
            if verbose:
                print("%s: %s" %(os.path.basename(sbml_file), db_ref))
            if db_ref.lower() != "metacyc":
                if verbose:
                    print("Creating id mapping file: %s" %dict_file)
                convert_sbml_db.map_sbml(sbml_file, "reaction", "metacyc", dict_file, verbose=verbose, mnx_reac_file=mnx_rxn_path, mnx_chem_file=mnx_cpd_path)


def orthogroup_to_sbml(dict_data):
    """
    dict_orthogroup: global var
    """
    #dict_data = {'study_name':'', 'o_compare_name': '', sbml_template':'', 'output':''}
    sbml = dict_data['sbml']
    orthodata_path = dict_data['orthodata_path']
    study_name = dict_data['study_name']
    output = dict_data['output']
    verbose = dict_data['verbose']
    veryverbose = dict_data['veryverbose']
    #orthogroups = dict_data['orthogroups']

    if verbose:
        print('Create sbml for ' + study_name)

    if veryverbose:
        extract_orthofinder_verbose = veryverbose
    else:
        extract_orthofinder_verbose = False

    all_model_sbml = extract_orthofinder.get_sbml_files(sbml, workflow="aucome", verbose=extract_orthofinder_verbose)
    #if orthogroups:
    #    extract_orthofinder.orthogroups_to_sbml(orthodata_path, all_model_sbml, output, study_name, extract_orthofinder_verbose)
    #else:
    extract_orthofinder.orthologue_to_sbml(orthodata_path, all_model_sbml, output, study_name, extract_orthofinder_verbose)


def orthology_to_padmet(sbml, orthofinder_sbml_path, input_pwt_padmet,
                         database_path, output_padmet, orthodata_path,
                         orthofinder_filtered_path, filtering_threshold_list,
                        verbose, veryverbose):
    source_tool = "ORTHOFINDER"
    source_category = "ORTHOLOGY"

    if verbose:
        print('Create padmet from sbml for ' + sbml)
    if veryverbose:
        sbml_padmet_verbose = veryverbose
    else:
        sbml_padmet_verbose = False

    sbml_to_padmet.sbml_to_padmetSpec(orthofinder_sbml_path + '/' + sbml,
                                      input_pwt_padmet,
                                      padmetRef_file=database_path,
                                      output=output_padmet,
                                      source_tool=source_tool,
                                      source_category=source_category,
                                      verbose=sbml_padmet_verbose)


def addOrthologyInPadmet(study_id, orthodata_path, output_padmet, verbose):
    """
    Add orthologs information to a padmet file.
    Args:
        study_id (str): ID of the species which padmet will be analyzed
        orthodata_path (str): path to Orthologues files
        output_padmet (str): path to the output padmet file
        verbose (boolean): verbose
    """
    # Read orthologs files and create a dictionary containing orthologs
    # infomations.
    all_orgs = set()
    all_orthologue_files = []
    # Process of orthologues.
    for _path, _folders, _files in os.walk(orthodata_path):
        for _folder in _folders:
            all_orgs.add(_folder.replace("Orthologues_","").lower())
        for _file in _files:
            _filename = os.path.splitext(_file)[0]
            if _filename.startswith(study_id):
                all_orthologue_files.append(os.path.join(_path,_file))
        dict_orthologues = {}

    for org in all_orgs:
        dict_orthologues[org] = dict()

    #dict_orthologue: k = org_id, v = dict: k = gene_id, v = dict: k = org_id, v = set of gene orthologue id
    for orthologue_file in [i for i in all_orthologue_files if "__v__" in i]:
        with open(orthologue_file, 'r') as csvfile:
            reader = csv.DictReader(csvfile, delimiter = "\t")
            orgs = list(reader.fieldnames)
            orgs.remove('Orthogroup')
            org_A, org_B = orgs
            for row in reader:
                gene_ids_A = [gene_id.split("_isoform")[0] for gene_id in row[org_A].split(", ")]
                gene_ids_B = [gene_id.split("_isoform")[0] for gene_id in row[org_B].split(", ")]
                for gene_id_A in gene_ids_A:
                    if gene_id_A not in dict_orthologues[org_A.lower()].keys():
                        dict_orthologues[org_A.lower()][gene_id_A] = dict()
                    dict_orthologues[org_A.lower()][gene_id_A][org_B.lower()] = set(gene_ids_B)

    padmet_file = os.path.basename(output_padmet)
    if verbose:
        print("%s..."%padmet_file)
    org_id = os.path.splitext(padmet_file)[0]

    org_id = org_id.lower()
    padmet = PadmetSpec(output_padmet)
    for linked_rlt in [rlt for rlt in padmet.getAllRelation() if rlt.type == "is_linked_to"]:
        gene_id = linked_rlt.id_out
        for index, src in enumerate(linked_rlt.misc["SOURCE:ASSIGNMENT"]):
            if 'GENOME:' not in src:
                ortho_org_id = src.replace("OUTPUT_ORTHOFINDER_FROM_","").split(':')[0]
                ortho_org_id_low = ortho_org_id.lower()
                org_id_orthologues = dict_orthologues[org_id][gene_id]
                ortho_genes_id = ";".join(org_id_orthologues[ortho_org_id_low])
                new_src = "OUTPUT_ORTHOFINDER_FROM_%s:%s"%(ortho_org_id, ortho_genes_id)
                linked_rlt.misc["SOURCE:ASSIGNMENT"][index] = new_src
    padmet.generateFile(output_padmet)


def filter_propagation(padmet_folder, output_folder, aucome_pool, filtering_threshold_list, union=None, intersection=None, verbose=None):
    propagation_to_remove_file = os.path.join(output_folder, "propagation_to_remove.tsv")
    reactions_to_remove_file = os.path.join(output_folder, 'reactions_to_remove.tsv')
    padmet_output_folder = output_folder

    if verbose:
        print("Extracting all the relations gene-reaction...")
    dict_rxn_orgs_genes, dict_rxn_ec = extractRGL(padmet_folder, aucome_pool)
    if verbose:
        print("Extracting all the gene propagations...")
    dict_rxn_org_gene_propagation = extractPropagation(dict_rxn_orgs_genes)
    if verbose:
        print("Writing the file propagation_to_remove...")
    dict_rxn_org_gene_propag_to_remove = extractPropagationToRemove(dict_rxn_org_gene_propagation, output=propagation_to_remove_file, orthology_threshold_list=filtering_threshold_list, union=union, intersection=intersection)
    if verbose:
        print("Cleaning the Padmet files and writing the reactions_to_remove_file file...")
    cleanPadmet(dict_rxn_org_gene_propag_to_remove, dict_rxn_ec, padmet_folder,
                padmet_output_folder, reactions_to_remove_file, aucome_pool)


def spontaneous_result_dict(first_dict, second_dict):
    for rxn_id in second_dict:
        if rxn_id not in first_dict:
            first_dict[rxn_id] = dict()
        for org_id in second_dict[rxn_id]:
            if org_id not in first_dict[rxn_id]:
                first_dict[rxn_id][org_id] = dict()
            for gene_id in second_dict[rxn_id][org_id]:
                if gene_id not in first_dict[rxn_id][org_id]:
                    first_dict[rxn_id][org_id][gene_id] = second_dict[rxn_id][org_id][gene_id]

def extractRGL(padmet_folder, aucome_pool):
    """
    extract reactions genes relations.
    It reads all Padmet files in padmet_folder, then it creates three
    dictionaries: dict_rnx_orgs_genes, dict_rnx_ec, dict_org_genes.
    dict_rxn_org_gene: reaction & organism & assigned genes & orthologuous genes.
    dict_rxn_ec[rxn_id] = ec, simple dictionary
    return dict {rxn_id:{org_id:{'FROM-PTOOL': bool, gene_id:set of sources (ex ortho_org_id:ortho_genes)}}}
    """
    dict_rxn_orgs_genes = {}
    dict_rxn_ec = {}
    multiprocessing_datas = []
    for padmet_file in next(os.walk(padmet_folder))[2]:
        multiprocessing_datas.append([padmet_file, padmet_folder])

    multiprocessing_results = aucome_pool.starmap(mp_extractRGL, multiprocessing_datas)

    for multiprocessing_result in multiprocessing_results:
        spontaneous_result_dict(dict_rxn_orgs_genes, multiprocessing_result[0])
        dict_rxn_ec.update(multiprocessing_result[1])

    return dict_rxn_orgs_genes, dict_rxn_ec


def mp_extractRGL(padmet_file, padmet_folder):
    padmet_path = os.path.join(padmet_folder, padmet_file)
    org_id = os.path.splitext(padmet_file)[0].upper()
    padmet = PadmetSpec(padmet_path)
    dict_rxn_orgs_genes = {}
    dict_rxn_ec = {}
    for node in padmet.dicOfNode.values():
        if node.type == "reaction":
            rxn_id = node.id
            if rxn_id not in dict_rxn_orgs_genes:
                dict_rxn_orgs_genes[rxn_id] = dict()

            if org_id not in dict_rxn_orgs_genes[rxn_id]:
                dict_rxn_orgs_genes[rxn_id][org_id] = dict()
            if 'EC-NUMBER' in node.misc:
                dict_rxn_ec[rxn_id] = node.misc['EC-NUMBER'][0]
            #rxn_id = "PHOSGLYPHOS-RXN"
            from_ptool = False
            for is_linked_rlt in [rlt for rlt in padmet.dicOfRelationIn[rxn_id] if rlt.type == "is_linked_to"]:
                gene_id = is_linked_rlt.id_out
                all_sources = {src.replace("OUTPUT_ORTHOFINDER_FROM_","")  for src in is_linked_rlt.misc["SOURCE:ASSIGNMENT"]}
                if any (src for src in all_sources if src.startswith("GENOME")):
                    from_ptool = True
                dict_rxn_orgs_genes[rxn_id][org_id][gene_id] = all_sources
            dict_rxn_orgs_genes[rxn_id][org_id]["FROM-PTOOL"] = from_ptool

    return dict_rxn_orgs_genes, dict_rxn_ec


def extractPropagation(dict_rxn_orgs_genes):
    """
    Propagations are extracted from Padmet networks. Then propagations
    are split bewteen two groups. Those who are orthologues to gene-reaction
    associations that come from Pathway Tools, and those whose associations
    do not come from Pathway Tools.
    """
    dict_rxn_org_gene_propagation = {}
    for rxn_id, rxn_data in dict_rxn_orgs_genes.items():
        dict_rxn_org_gene_propagation[rxn_id] = dict()
        for org_id, org_data in rxn_data.items():
            if org_id not in dict_rxn_org_gene_propagation[rxn_id].keys():
                dict_rxn_org_gene_propagation[rxn_id][org_id] = dict()
            #because of genes without gene link sources
            if len(org_data.keys()) > 1:
                for gene_id, gene_data in org_data.items():
                    if gene_id != "FROM-PTOOL":
                        gene_id = gene_id
                        if any (src for src in gene_data if src.startswith("GENOME")):
                            is_from_ptool = True
                        else:
                            is_from_ptool = False
                        for src in gene_data:
                            if not src.startswith("GENOME:"):
                                ortho_org_id, ortho_genes_ids = src.split(":")
                                ortho_genes_ids = ortho_genes_ids.split(";")
                                if ortho_org_id not in dict_rxn_org_gene_propagation[rxn_id].keys():
                                    dict_rxn_org_gene_propagation[rxn_id][ortho_org_id] = dict()
                                for ortho_gene_id in ortho_genes_ids:
                                    if gene_id in dict_rxn_org_gene_propagation[rxn_id][org_id]:
                                        dict_rxn_org_gene_propagation[rxn_id][org_id][gene_id]["propagation_from_ptool"].add((ortho_org_id, ortho_gene_id))
                                    else:
                                        dict_rxn_org_gene_propagation[rxn_id][org_id][gene_id] = {"propagation_to_ptool":set(),"propagation_to_not_ptool":set(), "propagation_from_ptool": set([(ortho_org_id, ortho_gene_id)])}

                                    if ortho_gene_id not in dict_rxn_org_gene_propagation[rxn_id][ortho_org_id].keys():
                                        dict_rxn_org_gene_propagation[rxn_id][ortho_org_id][ortho_gene_id] = {"propagation_to_ptool":set(),"propagation_to_not_ptool":set(), "propagation_from_ptool": set()}
                                    if is_from_ptool:
                                        dict_rxn_org_gene_propagation[rxn_id][ortho_org_id][ortho_gene_id]["propagation_to_ptool"].add((org_id, gene_id))
                                    else:
                                        dict_rxn_org_gene_propagation[rxn_id][ortho_org_id][ortho_gene_id]["propagation_to_not_ptool"].add((org_id, gene_id))

    return dict_rxn_org_gene_propagation


def extractPropagationToRemove(dict_rxn_org_gene_propagation, output,
                               ptool_threshold=0,
                               orthology_threshold_list=[0.05], union=None,
                               intersection=None):
    """
    Using ptool_threshold and orthology_threshold, this function select the 
    propagations to remove. These propagation are written in 
    propagation_to_remove.tsv.
    """
    header = ["reaction_id", "org_id", "gene_id"]
    dict_rxn_org_gene_propag_to_remove = dict()
    maximum = 5
    with open(output, 'w') as csvfile:
        writer = csv.DictWriter(csvfile, header, delimiter="\t")
        writer.writeheader()
        for rxn_id ,rxn_data in dict_rxn_org_gene_propagation.items():
            nb_org_prop = len(rxn_data.keys())-1
            if nb_org_prop:
                for org_id, org_data in rxn_data.items():
                    gene_ids_to_remove = set()
                    gene_id_tmp = ""
                    for gene_id, gene_data in org_data.items():
                        count = 0
                        # At this moment filter is as 20/N with 0.05
                        for orthology_threshold in orthology_threshold_list:
                            inverse_orthology_threshold = 1/orthology_threshold
                            not_ptool_threshold = round(max(inverse_orthology_threshold/nb_org_prop, orthology_threshold*nb_org_prop),0)
                            if len({i[0] for i in gene_data["propagation_to_ptool"]}) <= ptool_threshold:
                                if len({i[0] for i in gene_data["propagation_to_not_ptool"]}) >= not_ptool_threshold:                                    
                                    if intersection:
                                        count+=1 
                                        gene_id_tmp = gene_id
                                    elif not gene_id in gene_ids_to_remove:
                                        gene_ids_to_remove.add(gene_id)
                                        dict_rxn_org_gene_propag_to_remove = remove_gene(gene_ids_to_remove, dict_rxn_org_gene_propag_to_remove, rxn_id, org_id, writer)
                        if intersection and count == maximum:
                            gene_ids_to_remove.add(gene_id)
                            dict_rxn_org_gene_propag_to_remove = remove_gene(gene_ids_to_remove, dict_rxn_org_gene_propag_to_remove, rxn_id, org_id, writer)
    return dict_rxn_org_gene_propag_to_remove


# This function is called in extractPropagationToRemove().
def remove_gene(gene_ids_to_remove, dict_rxn_org_gene_propag_to_remove, rxn_id,
                org_id, writer):
    if gene_ids_to_remove:
        genes_ids = ";".join(gene_ids_to_remove)
        if rxn_id in dict_rxn_org_gene_propag_to_remove:
            dict_rxn_org_gene_propag_to_remove[rxn_id][org_id] = gene_ids_to_remove
        else:
            dict_rxn_org_gene_propag_to_remove[rxn_id] = {org_id: gene_ids_to_remove}
            line = {"reaction_id": rxn_id, "org_id": org_id, "gene_id": genes_ids}
            writer.writerow(line)
    return dict_rxn_org_gene_propag_to_remove


def spontaneous_dict_org_rxn_clean(dict_org_rxn_clean, tmp_dict_org_rxn_clean,
                             dict_rxn_org_gene_propag_to_remove):
    for org_id in tmp_dict_org_rxn_clean:
        if org_id not in dict_org_rxn_clean:
            dict_org_rxn_clean[org_id] = dict()
        for rxn_id in tmp_dict_org_rxn_clean[org_id]:
            if rxn_id in dict_rxn_org_gene_propag_to_remove.keys():
                if rxn_id not in dict_org_rxn_clean[org_id]:
                    dict_org_rxn_clean[org_id][rxn_id] = dict()
                for gene_id in tmp_dict_org_rxn_clean[org_id][rxn_id]:
                    if gene_id not in dict_org_rxn_clean[org_id][rxn_id]:
                        dict_org_rxn_clean[org_id][rxn_id][gene_id] = tmp_dict_org_rxn_clean[org_id][rxn_id][gene_id]


def create_dict_org_rxn_clean(padmet_file, padmet_folder,
                              dict_rxn_org_gene_propag_to_remove):
    padmet_path = os.path.join(padmet_folder, padmet_file)
    org_id = os.path.splitext(padmet_file)[0].upper()
    padmet = PadmetSpec(padmet_path)
    dict_org_rxn_clean = dict()
    dict_org_rxn_clean[org_id] = dict()
    for rxn_id in [node.id for node in padmet.dicOfNode.values() if node.type == "reaction" and node.id in dict_rxn_org_gene_propag_to_remove.keys()]:
        dict_org_rxn_clean[org_id][rxn_id] = dict()
        for is_linked_rlt in [rlt for rlt in padmet.dicOfRelationIn[rxn_id] if rlt.type == "is_linked_to"]:
            gene_id = is_linked_rlt.id_out
            all_sources = {src.replace("OUTPUT_ORTHOFINDER_FROM_","")  for src in is_linked_rlt.misc["SOURCE:ASSIGNMENT"]}
            new_sources = set()
            for src in all_sources:
                if src.startswith("GENOME:"):
                    new_sources.add(src)
                else:
                    ortho_org_id, ortho_genes_ids = src.split(":")
                    ortho_genes_ids = set(ortho_genes_ids.split(";"))
                    if ortho_org_id not in dict_rxn_org_gene_propag_to_remove[rxn_id].keys():
                        new_sources.add(src)
                    else:
                        new_ortho_genes_ids = set()
                        for ortho_gene_id in ortho_genes_ids:
                            if ortho_gene_id not in dict_rxn_org_gene_propag_to_remove[rxn_id][ortho_org_id]:
                                new_ortho_genes_ids.add(ortho_gene_id)
                        if new_ortho_genes_ids:
                            new_src = "%s:%s"%(ortho_org_id, ";".join(new_ortho_genes_ids))
                            new_sources.add(new_src)
            dict_org_rxn_clean[org_id][rxn_id][gene_id] = new_sources

    return dict_org_rxn_clean


# This function is called in the cleanPadmet() function. 
def delete_propagation(padmet_file, padmet_folder, dict_org_rxn_clean,
                       output_folder):
    padmet_path = os.path.join(padmet_folder, padmet_file)
    org_id = os.path.splitext(padmet_file)[0].upper()
    padmet = PadmetSpec(padmet_path)
    output = os.path.join(output_folder, padmet_file)
    nb_rxn_removed = 0
    for rxn_id, rxn_data in dict_org_rxn_clean[org_id].items():
        if any(rxn_data.keys()):
            if not any(rxn_data.values()):
                nb_rxn_removed += 1
                padmet.delNode(rxn_id)
            else:
                for gene_id, gene_data in rxn_data.items():
                    is_linked_rlt = [rlt for rlt in padmet.dicOfRelationOut[gene_id] if rlt.id_in == rxn_id and rlt.id_out == gene_id][0]
                    if not gene_data:
                        #remove relation
                        padmet._delRelation(is_linked_rlt)
                    else:
                        #update relation.misc[src:assgn], /!\ MAJ et source edition
                        is_linked_rlt.misc["SOURCE:ASSIGNMENT"] = ["OUTPUT_ORTHOFINDER_FROM_%s"%src for src in gene_data]
    print("Removing %s in %s"%(nb_rxn_removed, org_id))
    padmet.generateFile(output)


def cleanPadmet(dict_rxn_org_gene_propag_to_remove, dict_rxn_ec, padmet_folder,
                output_folder, reactions_to_remove_file, aucome_pool):
    """
    It cleans the Padmet files and it writes the reactions_to_remove_file file.
    """
    dict_org_rxn_clean = dict()
    create_multiprocessing_datas = []
    for padmet_file in next(os.walk(padmet_folder))[2]:
        create_multiprocessing_datas.append([padmet_file, padmet_folder, dict_rxn_org_gene_propag_to_remove])

    multiprocessing_results = aucome_pool.starmap(create_dict_org_rxn_clean, create_multiprocessing_datas)

    for multiprocessing_result in multiprocessing_results:
        spontaneous_dict_org_rxn_clean(dict_org_rxn_clean, multiprocessing_result, dict_rxn_org_gene_propag_to_remove)

    remove_multiprocessing_datas = []
    for padmet_file in next(os.walk(padmet_folder))[2]:
        remove_multiprocessing_datas.append([padmet_file, padmet_folder, dict_org_rxn_clean, output_folder])

    aucome_pool.starmap(delete_propagation, remove_multiprocessing_datas)

    with open(reactions_to_remove_file, 'w') as csvfile:
        header = ["org_id","reaction_id", "ec-number", "gene_id"]
        writer = csv.DictWriter(csvfile, header, delimiter="\t")
        writer.writeheader()
        list_count = []
        for org_id, org_data in dict_org_rxn_clean.items():
            to_remove = list()
            for rxn_id, rxn_data in org_data.items():
                if any(rxn_data.keys()) and not any(rxn_data.values()):
                    to_remove.append(rxn_id)
                    if rxn_id in dict_rxn_ec:
                        ec = dict_rxn_ec[rxn_id]
                    else:
                        ec = ''
                    gene_id = dict_org_rxn_clean[org_id][rxn_id]
                    line = {"org_id":org_id, "reaction_id":rxn_id,
                            "ec-number":ec, "gene_id":";".join(gene_id)}
                    writer.writerow(line)
                    list_count.append(rxn_id)
    return dict_org_rxn_clean



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import subprocess

from padmet.classes import PadmetSpec

def test_aucome():
    # Set working folder.
    aucome_set_working_folder = ['aucome', '--setWorkingFolder', os.getcwd()]
    subprocess.call(aucome_set_working_folder)
    # Check genbank and create fasta.
    aucome_cmd_check = ['aucome', 'check', '--run=test']
    subprocess.call(aucome_cmd_check)
    # Run Pathway-Tools.
    aucome_cmd_pwt = ['aucome', 'reconstruction', '--run=test']
    subprocess.call(aucome_cmd_pwt)
    # Run Orthofinder.
    aucome_cmd_ortho = ['aucome', 'orthology', '--run=test']
    subprocess.call(aucome_cmd_ortho)
    # Spontaneous all networks.
    aucome_cmd_spontaneous = ['aucome', 'draft', '--run=test']
    subprocess.call(aucome_cmd_spontaneous)

    padmetSpec = PadmetSpec('test/networks/fatty_acid_beta_oxydation_I_1.padmet')
    expected_fabo_reactions = [node.id for node in padmetSpec.dicOfNode.values() if node.type == "reaction"]
    fabo_reactions = ["ACYLCOASYN-RXN", "ACYLCOADEHYDROG-RXN", "ENOYL-COA-DELTA-ISOM-RXN", "ENOYL-COA-HYDRAT-RXN",
                    "OHBUTYRYL-COA-EPIM-RXN", "OHACYL-COA-DEHYDROG-RXN", "KETOACYLCOATHIOL-RXN"]

    assert set(fabo_reactions).issubset(set(expected_fabo_reactions))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome workflow --run=ID [--sequence_search_prg=STR] [--keep-tmp] [--cpu=INT] [-v] [--vv] [--filtering] [--threshold=FLOAT] [--union] [--intersection]

options:
    --run=ID    Pathname to the comparison workspace.
    --sequence_search_prg=STR    Sequence search program for Orthofinder [Default: diamond].
        Options: blast, mmseqs, blast_gz, diamond
    --keep-tmp    Keep temporary file (especially sequence of predicted gene linked to reaction).
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
    -v     Verbose.
    --vv    Very verbose.
    --filtering     Use a filter to limit propagation, by default it is 0.05, if you want to modify the value use --threshold.
    --threshold=FLOAT     Threshold of the filter to limit propagation to use with the --filtering argument.
    --union          Use the union filter between five threshold values [0.01, 0.05, 0.1, 0.15, 0.2] to limit propagation, to use with the --filtering argument.
    --intersection   Use the intersection filter between five threshold values [0.01, 0.05, 0.1, 0.15, 0.2] to limit propagation, to use with the --filtering argument.
"""
import aucome
import docopt
import sys
import time


def command_help():
    print(docopt.docopt(__doc__))


def workflow_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    sequence_search_prg = args['--sequence_search_prg']
    cpu = args['--cpu']
    keep_tmp = args['--keep-tmp']
    verbose = args['-v']
    veryverbose = args['--vv']
    filtering = args['--filtering']
    threshold = args['--threshold']
    union = args['--union']
    intersection = args['--intersection']
    filtering_threshold_list = []
    
    if filtering:
        if threshold:
            try:
                filtering_threshold_list.append(float(threshold))
            except:
                sys.exit('filtering_threshold value must be a float between 0 and 1.')       
        else:
            filtering_threshold_list.append(0.05)      
        if union or intersection:
            filtering_threshold_list = [0.01, 0.05, 0.1, 0.15, 0.2]
    else:
        if threshold:
            sys.exit('--threshold must be used with --filtering.')
        if union:
            sys.exit('--union must be used with --filtering.')
        if intersection:
            sys.exit('--intersection must be used with --filtering.')
            
    if cpu:
        nb_cpu_to_use = int(cpu)
    else:
        nb_cpu_to_use = 1

    if veryverbose and not verbose:
        verbose = veryverbose

    run_workflow(run_id, nb_cpu_to_use, sequence_search_prg, filtering_threshold_list, union, intersection, keep_tmp, verbose, veryverbose)


def run_workflow(run_id, nb_cpu_to_use, sequence_search_prg, filtering_threshold_list, union, intersection, keep_tmp, verbose, veryverbose=None):
    if verbose:
        print('--- Running workflow ---')
    workflow_start_time = time.time()

    aucome.check.run_check(run_id, nb_cpu_to_use, verbose, veryverbose)

    aucome.reconstruction.run_reconstruction(run_id, nb_cpu_to_use, verbose, veryverbose)

    aucome.orthology.run_orthology(run_id, sequence_search_prg, nb_cpu_to_use, filtering_threshold_list, union, intersection, verbose, veryverbose)

    aucome.structural.run_structural(run_id, keep_tmp, nb_cpu_to_use, verbose)

    aucome.spontaneous.run_spontaneous(run_id, nb_cpu_to_use, verbose, veryverbose)

    workflow_end_time = (time.time() - workflow_start_time)
    integer_part, decimal_part = str(workflow_end_time).split('.')
    workflow_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- workflow step done in: %ss ---" %workflow_time)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome structural --run=ID [--keep-tmp] [--cpu=INT] [-v]

options:
    --run=ID    Pathname to the comparison workspace.
    --keep-tmp    Keep temporary file (especially sequence of predicted gene linked to reaction).
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
    -v     Verbose.
"""

import docopt
import time

from padmet.utils.exploration import prot2genome

from aucome.utils import parse_config_file


def command_help():
    print(docopt.docopt(__doc__))


def structural_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    keep_tmp = args['--keep-tmp']
    verbose = args['-v']

    if args["--cpu"]:
        nb_cpu_to_use = int(args["--cpu"])
    else:
        nb_cpu_to_use = 1

    run_structural(run_id, keep_tmp, nb_cpu_to_use, verbose)


def run_structural(run_id, keep_tmp, nb_cpu_to_use, verbose):
    if verbose:
        print('--- Running structural check step ---')
    structural_start_time = time.time()

    config_data = parse_config_file(run_id)
    database_path = config_data['database_path']

    prot2genome.fromAucome(run_id, nb_cpu_to_use, database_path, blastp=True, tblastn=True, exonerate=True, keep_tmp=keep_tmp, debug=False)

    structural_end_time = (time.time() - structural_start_time)
    integer_part, decimal_part = str(structural_end_time).split('.')
    structural_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- structural step done in: %ss ---" %structural_time)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Some utils function.
"""

import configparser
import os
import sys


def parse_config_file(run_id):
    config_file_path = "{0}/config.txt".format(run_id)

    if not os.path.exists(config_file_path):
        sys.exit('Error no existing config file: ' + config_file_path)

    config = configparser.ConfigParser()
    config.read(config_file_path)

    #DATABASE_PATHS
    database_path = config.get('DATABASE_PATHS','database_ref_path')
    #PATHS_IN_RUN
    studied_organisms_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','studied_organisms_path'))
    orthology_based_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','orthology_based_path'))
    orthofinder_wd_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','orthofinder_wd_path'))
    orthofinder_sbml_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','orthofinder_sbml_path'))
    orthofinder_padmet_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','orthofinder_padmet_path'))
    orthofinder_filtered_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','orthofinder_filtered_path'))
    annotation_based_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','annotation_based_path'))
    pgdb_from_annotation_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','pgdb_from_annotation_path'))
    padmet_from_annotation_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','padmet_from_annotation_path'))
    sbml_from_annotation_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','sbml_from_annotation_path'))
    log_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','log_path'))
    analysis_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','analysis_path'))
    analysis_group_file_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','analysis_group_file_path'))
    networks_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','networks_path'))
    padmet_from_networks_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','padmet_from_networks_path'))
    sbml_from_networks_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','sbml_from_networks_path'))
    structural_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','structural_path'))
    structural_padmets_path = "{0}/{1}".format(run_id, config.get('PATHS_IN_RUN','structural_padmets_path'))

    #TOOL_PATHS
    orthofinder_bin_path = config.get('TOOL_PATHS','orthofinder_bin_path')
    #VAR
    study_from_annot_prefix = config.get('VAR','study_from_annot_prefix')

    config_data = {'database_path': database_path,
                    'studied_organisms_path': studied_organisms_path,
                    'orthology_based_path': orthology_based_path, 'orthofinder_wd_path': orthofinder_wd_path,
                    'orthofinder_sbml_path': orthofinder_sbml_path, 'orthofinder_padmet_path': orthofinder_padmet_path,
                    'orthofinder_filtered_path': orthofinder_filtered_path,
                    'annotation_based_path': annotation_based_path, 'pgdb_from_annotation_path': pgdb_from_annotation_path,
                    'padmet_from_annotation_path': padmet_from_annotation_path, 'sbml_from_annotation_path': sbml_from_annotation_path,
                    'sbml_from_annotation_path': sbml_from_annotation_path, 'log_path': log_path,
                    'networks_path': networks_path, 'orthofinder_bin_path': orthofinder_bin_path,
                    'structural_path': structural_path, 'structural_padmets_path': structural_padmets_path,
                    'study_from_annot_prefix': study_from_annot_prefix, 'analysis_path': analysis_path,
                    'padmet_from_networks_path': padmet_from_networks_path, 'sbml_from_networks_path': sbml_from_networks_path,
                    'analysis_group_file_path': analysis_group_file_path}

    return config_data



"""
usage:
    aucome --init=ID [-v]
    aucome <command> [<args>...]
    aucome -R --run=ID
    aucome --installPWT=PWT_path [--ptools=ptools_path]
    aucome --uninstallPWT

options:
    -h --help     Show help.
    -R     Give access from container.
    --init=ID    Create folder structure.
    --run=ID    Pathname to the comparison workspace.
    -v     Verbose.

The subcommands are:
    check    Check inputs validity
    reconstruction    Run Pathway Tools
    orthology    Run Orthofinder for crossing orthology between species
    structural    Run check of the structural annotation
    spontaneous    Spontaneous all networks (from Pathway Tools, Orthology and Structural)

    workflow    Run Check, Pathway Tools, Orthofinder and Merging of all networks
    analysis    Analyze results
    compare    Compare group of species

See 'aucome <command> -h' for more information on a specific command.
"""
import configparser
import csv
import docopt
import logging
import mpwt
import os
import pkg_resources
import re
import requests
import shutil
import subprocess
import sys

import aucome

logging.basicConfig(format='%(message)s', level=logging.CRITICAL)
logger = logging.getLogger(__name__)
logging.getLogger("aucome").setLevel(logging.DEBUG)


def main(args=None):
    args = docopt.docopt(__doc__, options_first=True)

    command = args.pop('<command>')
    command_args = args.pop('<args>')


    if args["--init"]:
        run_id = args["--init"]
        create_run(run_id)
        chmod_cmds = ["chmod", "-R", "777", run_id]
        subprocess.call(chmod_cmds)
        return

    # Add permission to all folder in run_id, usefull because all cmd exec from container are root based.
    if args['-R']:
        run_id = args["--run"]
        chmod_cmds = ["chmod", "-R", "777", run_id]
        subprocess.call(chmod_cmds)
        return

    if args['--installPWT']:
        installing_pwt(args['--installPWT'], args['--ptools'])
        return

    if args['--uninstallPWT']:
        uninstalling_pwt()
        return

    if command:
        if command not in ['workflow', 'check', 'reconstruction', 'orthology', 'spontaneous', 'structural', 'analysis', 'compare']:
            sys.exit(command + ' not a valid command: workflow, check, reconstruction, orthology, spontaneous, structural, analysis, compare.')

        if '-h' in command_args:
            getattr(aucome, command).command_help()
            sys.exit()

        # Add command to command_args to be parse by docopt.
        command_args.insert(0,command)

        if command == 'workflow':
            aucome.workflow.workflow_parse_args(command_args)

        elif command == 'check':
            aucome.check.check_parse_args(command_args)

        elif command == 'reconstruction':
            aucome.reconstruction.reconstruction_parse_args(command_args)
        
        elif command == 'orthology':
            aucome.orthology.orthology_parse_args(command_args)

        elif command == 'structural':
            aucome.structural.structural_parse_args(command_args)

        elif command == 'spontaneous':
            aucome.spontaneous.spontaneous_parse_args(command_args)

        elif command == 'analysis':
            aucome.analysis.analysis_parse_args(command_args)

        elif command == 'compare':
            aucome.compare.compare_parse_args(command_args)


def create_run(run_id):
    """
    Create a run folder
    """
    if os.path.isdir('{0}'.format(run_id)):
        print("Run '%s' already exist, remove this folder manually before" %run_id)
    else:
        print('creating Run %s' %run_id)
        os.mkdir('{0}'.format(run_id))
        all_folders = [['studied_organisms'], ['networks'], ['orthology_based'],\
                       ['orthology_based', '0_Orthofinder_WD'], \
                        ['orthology_based', '1_sbml_orthology'], \
                       ['orthology_based', '2_padmet_orthology'], ['orthology_based', '3_padmet_filtered'], ['annotation_based'],\
                       ['annotation_based', 'PGDBs'], ['annotation_based', 'PADMETs'],\
                       ['annotation_based', 'SBMLs'], ['analysis'], ['logs'],\
                       ['networks', 'PADMETs'], ['networks', 'SBMLs'],
                       ['structural_check'], ['structural_check', '0_specifics_reactions'],
                       ['structural_check', '1_blast_results'], ['structural_check', '1_blast_results', 'analysis'], ['structural_check', '1_blast_results', 'tmp'],
                       ['structural_check', '1_blast_results', 'reactions_sequences'],
                       ['structural_check', '2_reactions_to_add'], ['structural_check', '3_PADMETs']]
        for folder in all_folders:
            folder_path = os.path.join(run_id, *folder)
            print('creating folder {0}'.format(folder_path))
            os.mkdir(folder_path)

        group_template_path = os.path.join(run_id, *['analysis', 'group_template.tsv'])
        studied_organisms_path = os.path.join(run_id, 'studied_organisms')
        with open(group_template_path, 'w') as group_file:
            group_writer = csv.writer(group_file, delimiter='\t')
            group_writer.writerow(['all', *os.listdir(studied_organisms_path)])
        config_file_path = os.path.join(run_id, 'config.txt')
        create_config_file(config_file_path, run_id)


def create_config_file(config_file_path, run_id):
    config = configparser.RawConfigParser()
    config.add_section('DATABASE_PATHS')
    config.set('DATABASE_PATHS', 'database_ref_path', '/home/database/BIOCYC/METACYC/23.5/metacyc_23.5.padmet')
    config.add_section('PATHS_IN_RUN')
    config.set('PATHS_IN_RUN', 'run_id', run_id)
    config.set('PATHS_IN_RUN', 'studied_organisms_path', '/studied_organisms')
    config.set('PATHS_IN_RUN', 'orthology_based_path', '/orthology_based')
    config.set('PATHS_IN_RUN', 'orthofinder_wd_path', '/orthology_based/0_Orthofinder_WD')
    config.set('PATHS_IN_RUN', 'orthofinder_sbml_path', '/orthology_based/1_sbml_orthology')
    config.set('PATHS_IN_RUN', 'orthofinder_padmet_path', '/orthology_based/2_padmet_orthology')
    config.set('PATHS_IN_RUN', 'orthofinder_filtered_path', '/orthology_based/3_padmet_filtered')
    config.set('PATHS_IN_RUN', 'annotation_based_path', '/annotation_based')
    config.set('PATHS_IN_RUN', 'pgdb_from_annotation_path', '%(annotation_based_path)s/PGDBs')
    config.set('PATHS_IN_RUN', 'padmet_from_annotation_path', '%(annotation_based_path)s/PADMETs')
    config.set('PATHS_IN_RUN', 'sbml_from_annotation_path', '%(annotation_based_path)s/SBMLs')
    config.set('PATHS_IN_RUN', 'networks_path', '/networks')
    config.set('PATHS_IN_RUN', 'padmet_from_networks_path', '%(networks_path)s/PADMETs')
    config.set('PATHS_IN_RUN', 'sbml_from_networks_path', '%(networks_path)s/SBMLs')
    config.set('PATHS_IN_RUN', 'log_path', '/logs')
    config.set('PATHS_IN_RUN', 'analysis_path', '/analysis')
    config.set('PATHS_IN_RUN', 'analysis_group_file_path', '%(analysis_path)s/group_template.tsv')

    config.set('PATHS_IN_RUN', 'structural_path', '/structural_check')
    config.set('PATHS_IN_RUN', 'structural_specifics_reactions_path', '%(structural_path)s/0_specifics_reactions')
    config.set('PATHS_IN_RUN', 'structural_blast_results_path', '%(structural_path)s/1_blast_results')
    config.set('PATHS_IN_RUN', 'structural_reactions_to_add_path', '%(structural_path)s/2_reactions_to_add')
    config.set('PATHS_IN_RUN', 'structural_padmets_path', '%(structural_path)s/3_PADMETs')
    config.set('PATHS_IN_RUN', 'structural_blast_results_analysis_path', '%(structural_blast_results_path)s/analysis')
    config.set('PATHS_IN_RUN', 'structural_blast_results_tmp_path', '%(structural_blast_results_path)s/tmp')
    config.set('PATHS_IN_RUN', 'structural_blast_results_reactions_sequences_path', '%(structural_blast_results_path)s/reactions_sequences')

    config.add_section('TOOL_PATHS')
    config.set('TOOL_PATHS', 'orthofinder_bin_path', '/programs/OrthoFinder/orthofinder')
    config.add_section('VAR')
    config.set('VAR', 'study_from_annot_prefix', 'output_pathwaytools_')

    # Writing our configuration file to 'example.cfg'
    with open(config_file_path, 'w') as configfile:
        config.write(configfile)


def installing_pwt(pwt_path, input_ptools_local_path):
    """
    Install silently Pathway-Tools in /programs.
    After running this function you need to source the bashrc.
    Args:
        pwt_path (str): Path to the Pathway Tools installer
        input_ptools_local_path (str): Path to local data of Pathway Tools
    """
    ptools_local_path = '/root'
    if input_ptools_local_path:
        if os.path.isdir(input_ptools_local_path):
            ptools_local_path = input_ptools_local_path
        else:
            print(input_ptools_local_path + ' path does not exist, --ptools must be an existing path.')
            return

    cmd_chmods = ['chmod', 'u+x', pwt_path]
    cmd_installs = [pwt_path, '--InstallDir', '/programs/pathway-tools', '--PTOOLS_LOCAL_PATH', ptools_local_path,
                    '--InstallDesktopShortcuts', '0', '--mode', 'unattended']

    print(' '.join(cmd_chmods))
    subprocess.call(cmd_chmods)
    print(' '.join(cmd_installs))
    subprocess.call(cmd_installs)

    # Add Pathway-Tools in the PATH.
    cmd_echo = '''echo 'export PATH="$PATH:/programs/pathway-tools:"' >> ~/.bashrc'''
    print(cmd_echo)
    subprocess.call(cmd_echo, shell=True)

    print('Now you need to source your bash, run:')
    print('source ~/.bashrc')
    return


def uninstalling_pwt():
    """
    Uninstall Pathway-Tools and can delete ptools-local folder.
    """
    def ask_delete_ptools(ptools_path):
        yes_or_no = input('Delete ptools-local folder (y/n)?')
        if yes_or_no == 'y':
            shutil.rmtree(ptools_path)
            print('Uninstallation of Pahtway-Tools and ptools-local done!')
            return
        elif yes_or_no == 'n':
            print('Uninstallation of Pathway-Tools done!.')
            return
        else:
            print('Wrong command')
            ask_delete_ptools(ptools_path)

    ptools_path = mpwt.find_ptools_path()

    cmd_uninstall = ['/programs/pathway-tools/uninstall', '--mode', 'unattended']
    cmd_clean_bash = '''grep -v 'export PATH="$PATH:/programs/pathway-tools:"' ~/.bashrc > ~/temp.bashrc; mv ~/temp.bashrc ~/.bashrc'''

    print(' '.join(cmd_uninstall))
    subprocess.call(cmd_uninstall)

    print(cmd_clean_bash)
    subprocess.call(cmd_clean_bash, shell=True)

    if os.path.isdir('/root/AIC-prefs'):
        shutil.rmtree('/root/AIC-prefs')

    ask_delete_ptools(ptools_path)
    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome analysis --run=ID [--cpu=INT] [--pvclust] [-v]

options:
    --run=ID    Pathname to the comparison workspace.
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu). [default: 1]
    --pvclust     Run also pvclust to create another dendrogram of reaction.
    -v     Verbose.

"""

import csv
import docopt
import os
import sys
import time

from multiprocessing import Pool

from padmet.utils.connection import sbmlGenerator, padmet_to_padmet
from padmet.utils.exploration import compare_padmet, dendrogram_reactions_distance
from padmet.classes import PadmetRef

from aucome.utils import parse_config_file


def command_help():
    """
    Show help for analysis command.
    """
    print(docopt.docopt(__doc__))


def analysis_parse_args(command_args):
    """ Parse args from __main__.py

    Args:
        command_args (list): command arguments
    """
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    verbose = args['-v']
    nb_cpu_to_use = int(args["--cpu"])
    pvclust = args['--pvclust']

    run_analysis(run_id, nb_cpu_to_use, pvclust, verbose)


def run_analysis(run_id, nb_cpu_to_use, pvclust, verbose):
    """Create input data for creationf of reaction dendrogram tsv reactions files.

    Args:
        run_id (str): ID of the run
        nb_cpu_to_use (int): number of CPU for multiprocessing
        pvclust (boolean): use also pvclust to create reaction dendrogram
        verbose (boolean): verbose
    """
    if verbose:
        print('--- Running analysis step ---')
    analysis_start_time = time.time()
    config_data = parse_config_file(run_id)

    analysis_group_file_path = config_data['analysis_group_file_path']

    # Create list of dictionaries containing input data for multiprocessing.
    # As we have to give one argument after the function to pool().
    # Create one dictionary by group in the group_template.tsv file.
    group_data = []
    with open(analysis_group_file_path, 'r') as group_file:
        group_reader = csv.reader(group_file, delimiter='\t')
        for row in group_reader:
            group_name = row[0]
            groups = [org_name for org_name in row[1:] if org_name]
            analysis_on_group(group_name, groups, config_data, pvclust, nb_cpu_to_use, verbose)

    analysis_end_time = (time.time() - analysis_start_time)
    integer_part, decimal_part = str(analysis_end_time).split('.')
    analysis_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- analysis step done in: %ss ---" %analysis_time)


def analysis_on_group(group_name, groups, config_data, pvclust, nb_cpu_to_use, verbose):
    """Create reaction dendrogram and extract specific reactions using metabolic networks.

    Args:
        group_name (str): Name of the group from group_template.tsv.
        groups (list): All the species inside the group.
        config_data (dict): Dictionary with all configuration paths.
        pvclust (boolean): use also pvclust to create reaction dendrogram
        nb_cpu_to_use (int): number of CPU for multiprocessing
        verbose (bool): Verbose.
    """

    database_path = config_data['database_path']
    padmetRef = PadmetRef(database_path)
    padmet_from_networks_path = config_data['padmet_from_networks_path']
    analysis_path = config_data['analysis_path']

    all_padmet_path = [os.path.join(padmet_from_networks_path,name+".padmet") for name in groups ]
    group_analysis_path = analysis_path + '/' + group_name

    if not os.path.isdir(group_analysis_path):
        if len(groups) == 1:
            sys.exit('A group must contain more than one member.')

        for padmet_path in all_padmet_path:
            if not os.path.exists(padmet_path):
                org_name = os.path.splitext(os.path.basename(padmet_path))[0]
                sys.exit("Padmet file of organism %s from group %s not found in %s" %(org_name, group_name, padmet_from_networks_path))

        # Compare the padmet to create the reactions.tsv file needed to create the reaction dendrogram.
        compare_padmet.compare_padmet(padmet_path=",".join(all_padmet_path), output=group_analysis_path, padmetRef=padmetRef, verbose=verbose, number_cpu=nb_cpu_to_use)
        padmet_to_padmet.padmet_to_padmet(",".join(all_padmet_path), group_analysis_path + '/' + group_name + '_panmetabolism.padmet')
        sbmlGenerator.padmet_to_sbml(padmet=group_analysis_path + '/' + group_name + '_panmetabolism.padmet', output=group_analysis_path + '/' + group_name + '_panmetabolism.sbml', verbose=verbose)

        dendrogram_reactions_distance.reaction_figure_creation(reaction_file=group_analysis_path + '/reactions.tsv', output_folder=group_analysis_path + '/dendrogram_output',
                                                                padmetRef_file=database_path, pvclust=pvclust, verbose=verbose)

    else:
        print(group_analysis_path + ' already exists. Delete it if you want to relaunch the analysis.')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome compare --run=ID [--cpu=INT] [-v]

options:
    --run=ID    Pathname to the comparison workspace.
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu). [default: 1]
    -v     Verbose.

"""

import csv
import docopt
import matplotlib.pyplot as plt
import os
import pandas as pa
import time

from aucome.utils import parse_config_file
from padmet.classes.padmetRef import PadmetRef
from padmet.utils.exploration import compare_padmet, dendrogram_reactions_distance
from supervenn import supervenn


def command_help():
    """
    Show help for analysis command.
    """
    print(docopt.docopt(__doc__))


def compare_parse_args(command_args):
    """ Parse args from __main__.py

    Args:
        command_args (list): command arguments
    """
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    verbose = args['-v']

    if args["--cpu"]:
        nb_cpu_to_use = int(args["--cpu"])
    else:
        nb_cpu_to_use = 1

    run_compare(run_id, nb_cpu_to_use, verbose)


def run_compare(run_id, nb_cpu_to_use, verbose):
    """Compare the group specified by the user.

    Args:
        run_id (str): ID of the run
        nb_cpu_to_use (int): number of CPU for multiprocessing
        verbose (boolean): verbose
    """
    if verbose:
        print('--- Running compare step ---')
    compare_start_time = time.time()
    config_data = parse_config_file(run_id)

    analysis_path = config_data['analysis_path']
    analysis_group_file_path = config_data['analysis_group_file_path']
    compare_output_path = analysis_path + '/compare_group'

    database_path = config_data['database_path']
    padmet_from_networks_path = config_data['padmet_from_networks_path']

    # Create a dictionary containing the group name and the species inside the group.
    group_data = {}
    padmets = []
    with open(analysis_group_file_path, 'r') as group_file:
        group_reader = csv.reader(group_file, delimiter='\t')
        cluster_reactions = {}
        for row in group_reader:
            group_name = row[0]
            groups = [species for species in row[1:] if species != '']
            group_data[group_name] = groups
            if group_name != 'all':
                padmets.extend([padmet_from_networks_path + '/' + species + '.padmet' for species in groups])

    padmets = list(set(padmets))

    if not os.path.isdir(compare_output_path):
        os.mkdir(compare_output_path)

    padmetref = PadmetRef(database_path)
    # Create the reactions.tsv file needed to create dendrogram.
    padmet_path = ','.join(padmets)
    compare_padmet.compare_padmet(padmet_path=padmet_path, output=compare_output_path, padmetRef=padmetref, verbose=verbose)

    # Read the reactions.tsv file and remove the column unused.
    reactions_file = compare_output_path + '/' + 'reactions.tsv'
    reactions_dataframe = pa.read_csv(reactions_file, sep='\t')
    columns = [column for column in reactions_dataframe.columns if '(sep=;)' not in column and '_formula' not in column]
    reactions_dataframe = reactions_dataframe[columns].copy()
    reactions_dataframe.set_index('reaction', inplace=True)

    # For each group, extract the reactions present in its species to create supervenn sets.
    supervenn_sets = []
    supervenn_labels = []
    for group_name in group_data:
        if group_name != 'all':
            groups = group_data[group_name]
            reactions_temp = []
            for species in groups:
                species_reactions_dataframe = reactions_dataframe[reactions_dataframe[species] == 1]
                reactions_temp.extend(species_reactions_dataframe.index.tolist())
            supervenn_sets.append(set(reactions_temp))
            supervenn_labels.append(group_name)
            cluster_reactions[group_name] = set(reactions_temp)

    supervenn(supervenn_sets, supervenn_labels, chunks_ordering='occurrence', sets_ordering='minimize gaps')
    plt.savefig(compare_output_path + '/compare_group.png', bbox_inches='tight')
    plt.clf()

    dendrogram_reactions_distance.reaction_figure_creation(reactions_file, os.path.join(compare_output_path, "dendrogram_output"), padmetRef_file=database_path, verbose=verbose)

    compare_end_time = (time.time() - compare_start_time)
    integer_part, decimal_part = str(compare_end_time).split('.')
    compare_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- compare step done in: %ss ---" %compare_time)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome reconstruction --run=ID [--cpu=INT] [-v] [--vv]

options:
    --run=ID    Pathname to the comparison workspace.
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
    -v     Verbose.
    --vv    Very verbose.
"""

import docopt
import logging
import mpwt
import os
import time

from padmet.utils.connection import pgdb_to_padmet, sbmlGenerator

from aucome.utils import parse_config_file
from multiprocessing import Pool

logger = logging.getLogger('aucome')
logger.setLevel(logging.CRITICAL)
logging.getLogger("mpwt").setLevel(logging.CRITICAL)


def command_help():
    print(docopt.docopt(__doc__))


def reconstruction_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    verbose = args['-v']
    veryverbose = args['--vv']

    if args["--cpu"]:
        nb_cpu_to_use = int(args["--cpu"])
    else:
        nb_cpu_to_use = 1

    if veryverbose and not verbose:
        verbose = veryverbose

    run_reconstruction(run_id, nb_cpu_to_use, verbose, veryverbose)


def run_reconstruction(run_id, nb_cpu_to_use, verbose, veryverbose=None):
    if verbose:
        logger.setLevel(logging.DEBUG)
        logging.getLogger("mpwt").setLevel(logging.DEBUG)
        print('--- Running reconstruction step ---')
    start_time = time.time()
    config_data = parse_config_file(run_id)

    pgdb_from_annotation_path = config_data['pgdb_from_annotation_path']
    studied_organisms_path = config_data['studied_organisms_path']
    log_path = config_data['log_path']


    taxon_file = None
    if 'taxon_id.tsv' in set(next(os.walk(config_data['studied_organisms_path']))[2]):
        taxon_file = True

    mpwt.multiprocess_pwt(input_folder=studied_organisms_path,
                            output_folder=pgdb_from_annotation_path,
                            patho_inference=True,
                            flat_creation=True,
                            dat_extraction=True,
                            number_cpu=nb_cpu_to_use,
                            patho_log=log_path,
                            taxon_file=taxon_file,
                            verbose=verbose)

    if os.listdir(pgdb_from_annotation_path) == []:
        print('Pathway-Tools inference failed!')
        return

    create_padmet_sbml_from_pgdb(run_id, nb_cpu_to_use, verbose, veryverbose)

    end_time = (time.time() - start_time)
    integer_part, decimal_part = str(end_time).split('.')
    reconstruction_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- reconstruction step done in: %ss ---" %reconstruction_time)

def create_padmet_sbml_from_pgdb(run_id, nb_cpu_to_use, verbose, veryverbose):
    config_data = parse_config_file(run_id)

    padmet_from_annotation_path = config_data['padmet_from_annotation_path']
    study_from_annot_prefix = config_data['study_from_annot_prefix']
    sbml_from_annotation_path = config_data['sbml_from_annotation_path']
    database_path = config_data['database_path']
    pgdb_from_annotation_path = config_data['pgdb_from_annotation_path']

    aucome_pool = Pool(nb_cpu_to_use)

    all_study_name = set(next(os.walk(config_data['studied_organisms_path']))[1])

    all_study_pgdb = dict([(study_name, "{0}/{1}".format(pgdb_from_annotation_path, study_name))
                        if os.path.isdir("{0}/{1}".format(pgdb_from_annotation_path, study_name))
                        else (study_name, '')
                        for study_name in all_study_name])

    study_padmet_data = []
    for study_name in all_study_name:
        padmet_file = "{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name)
        pgdb_folder = all_study_pgdb[study_name]
        tmp_padmet_data = {'study_name': study_name, 'pgdb_folder': pgdb_folder, 'veryverbose': veryverbose,
                            'verbose': verbose, 'padmet_file': padmet_file, 'database_path': database_path}
        study_padmet_data.append(tmp_padmet_data)
    aucome_pool.map(create_padmet_from_pgdb, study_padmet_data)

    all_study_padmet = dict([(study_name, "{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name))
                            if os.path.isfile("{0}/{1}{2}.padmet".format(padmet_from_annotation_path, study_from_annot_prefix, study_name))
                            else (study_name, '')
                            for study_name in all_study_name])

    study_sbml_data = []
    for study_name in all_study_padmet:
        sbml_file = "{0}/{1}{2}.sbml".format(sbml_from_annotation_path, study_from_annot_prefix, study_name)
        padmet_file = all_study_padmet[study_name]
        tmp_sbml_data = {'sbml_file': sbml_file, 'padmet_file': padmet_file,
                         'study_name': study_name, 'verbose': verbose, 'veryverbose': veryverbose}
        study_sbml_data.append(tmp_sbml_data)
    aucome_pool.map(create_sbml, study_sbml_data)


def create_padmet_from_pgdb(tmp_padmet_data):
    study_name = tmp_padmet_data['study_name']
    pgdb_folder = tmp_padmet_data['pgdb_folder']
    verbose = tmp_padmet_data['verbose']
    padmet_file = tmp_padmet_data['padmet_file']
    database_path = tmp_padmet_data['database_path']
    veryverbose = tmp_padmet_data['veryverbose']

    if not os.path.isfile(padmet_file) and pgdb_folder:
        if verbose:
            print("Creating padmet from pgdb for %s" %study_name)
        pgdb_to_padmet.from_pgdb_to_padmet(pgdb_folder=pgdb_folder, padmetRef_file=database_path, source="genome", extract_gene=True, no_orphan=True, verbose=veryverbose, output_file=padmet_file)


def create_sbml(tmp_sbml_data):
    sbml_file = tmp_sbml_data['sbml_file']
    padmet_file = tmp_sbml_data['padmet_file']
    study_name = tmp_sbml_data['study_name']
    verbose = tmp_sbml_data['verbose']
    veryverbose = tmp_sbml_data['veryverbose']

    if not os.path.isfile(sbml_file) and padmet_file:
        if verbose:
            print("Creating sbml from padmet for %s" %study_name)
        sbmlGenerator.padmet_to_sbml(padmet=padmet_file, output=sbml_file, sbml_lvl=3, verbose=veryverbose)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='aucome',
      url='https://github.com/aureme/aucome',
      license='GPLv3+',
      description='Automatic Comparison of Metabolism',
      long_description=
      'AuCoMe is a Python3 workflow that aims at reconstructing homogeneous metabolic networks from genomes with heterogeneous levels of annotations. More information on usage and troubleshooting on Github: https://github.com/aureme/aucome',
      author='AuReMe',
      author_email='gem-aureme@inria.fr',
      python_requires='>=3.6',
      classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
          'Development Status :: 3 - Alpha',       
        # Audience
          'Intended Audience :: Science/Research',
          'Intended Audience :: Developers',
          'License :: OSI Approved',
          'Natural Language :: English',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          
        # Environnement, OS, languages
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7'
      ],
      packages=['aucome'],
      install_requires=['matplotlib', 'mpwt', 'padmet', 'rpy2==3.0.5',
                        'seaborn', 'supervenn', 'tzlocal'],      
      entry_points={
          'console_scripts': [
              'aucome = aucome.__main__:main'
          ]
      },
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
usage:
    aucome spontaneous --run=ID [--cpu=INT] [-v] [--vv]

options:
    --run=ID    Pathname to the comparison workspace.
    --cpu=INT     Number of cpu to use for the multiprocessing (if none use 1 cpu).
    -v     Verbose.
    --vv    Very verbose.
"""

import docopt
import os
import shutil
import sys
import time

from shutil import copyfile
from padmet.utils.connection import sbml_to_padmet, sbmlGenerator, padmet_to_padmet
from padmet.classes import PadmetRef, PadmetSpec
from aucome.utils import parse_config_file
from multiprocessing import Pool


def command_help():
    print(docopt.docopt(__doc__))


def spontaneous_parse_args(command_args):
    args = docopt.docopt(__doc__, argv=command_args)
    run_id = args['--run']
    verbose = args['-v']
    veryverbose = args['--vv']

    if args["--cpu"]:
        nb_cpu_to_use = int(args["--cpu"])
    else:
        nb_cpu_to_use = 1

    if veryverbose and not verbose:
        verbose = veryverbose

    run_spontaneous(run_id, nb_cpu_to_use, verbose, veryverbose)


def run_spontaneous(run_id, nb_cpu_to_use, verbose, veryverbose=None):
    if verbose:
        print('--- Running spontaneous step ---')
    spontaneous_start_time = time.time()
    aucome_pool = Pool(nb_cpu_to_use)

    config_data = parse_config_file(run_id)

    padmet_from_annotation_path = config_data['padmet_from_annotation_path']
    padmet_from_networks_path = config_data['padmet_from_networks_path']
    sbml_from_networks_path = config_data['sbml_from_networks_path']
    database_path = config_data['database_path']

    structural_padmets_path = config_data['structural_padmets_path']
    orthofinder_filtered_path = config_data['orthofinder_filtered_path']
    orthofinder_padmet_path = config_data['orthofinder_padmet_path']
    padmet_from_annotation_path = config_data['padmet_from_annotation_path']
    networks_path = config_data['networks_path']

    structural_padmets = [padmet for padmet in os.listdir(structural_padmets_path) if padmet.endswith('.padmet')]
    orthofinder_filtered_padmets = [padmet for padmet in os.listdir(orthofinder_filtered_path) if padmet.endswith('.padmet')]
    orthofinder_padmets = [padmet for padmet in os.listdir(orthofinder_padmet_path) if padmet.endswith('.padmet')]
    pathway_tools_padmets = [padmet for padmet in os.listdir(padmet_from_annotation_path) if padmet.endswith('.padmet')]

    if len(structural_padmets) > 0:
        padmets = [(padmet, structural_padmets_path + '/' + padmet) for padmet in structural_padmets]
    elif len(orthofinder_filtered_padmets) > 0:
        padmets = [(padmet, orthofinder_filtered_path + '/' + padmet) for padmet in orthofinder_filtered_padmets]
    elif len(orthofinder_padmets) > 0:
        padmets = [(padmet, orthofinder_padmet_path + '/' + padmet) for padmet in orthofinder_padmets]
    elif len(pathway_tools_padmets) > 0:
        padmets = [(padmet, padmet_from_annotation_path + '/' + padmet) for padmet in pathway_tools_padmets]
    else:
        sys.exit('No padmets have been created, run reconstruction or workflow.')

    study_draft_data = []
    for study_name, padmet_path in padmets:
        tmp_study_data = {'padmet_path': padmet_path, 'study_padmet': study_name, 'padmet_from_networks_path': padmet_from_networks_path,
                            'sbml_from_networks_path': sbml_from_networks_path, 'database_path': database_path,
                            'verbose': verbose, 'veryverbose': veryverbose}
        study_draft_data.append(tmp_study_data)
    aucome_pool.map(create_output, study_draft_data)

    aucome_pool.close()
    aucome_pool.join()

    padmet_to_padmet.padmet_to_padmet(padmet_from_networks_path, networks_path + '/panmetabolism.padmet', verbose=veryverbose)
    sbmlGenerator.padmet_to_sbml(padmet=networks_path + '/panmetabolism.padmet', output=networks_path + '/panmetabolism.sbml', verbose=veryverbose)

    spontaneous_end_time = (time.time() - spontaneous_start_time)
    integer_part, decimal_part = str(spontaneous_end_time).split('.')
    spontaneous_time = ".".join([integer_part, decimal_part[:3]])

    if verbose:
        print("--- spontaneous step done in: %ss ---" %spontaneous_time)


def add_spontaneous_reactions(padmet_path, padmet_ref_path, output_padmet_path, only_complete_pathways=True):
    number_spontaneous_reactions = 0

    padmetSpec = PadmetSpec(padmet_path)
    padmetRef = PadmetRef(padmet_ref_path)

    all_spontaneous_rxns = set([node.id for node in list(padmetRef.dicOfNode.values()) if node.type == "reaction" and "SPONTANEOUS" in node.misc])

    for spontaneous_rxn_id in all_spontaneous_rxns:
        in_pwys = set([rlt.id_out for rlt in padmetRef.dicOfRelationIn.get(spontaneous_rxn_id,None) if rlt.type == "is_in_pathway"])
        for pwy_id in in_pwys:
            if pwy_id in padmetSpec.dicOfNode.keys():
                padmet_ref_in_rxns = set([rlt.id_in for rlt in padmetRef.dicOfRelationOut.get(pwy_id,[]) if rlt.type == "is_in_pathway"])
                padmet_spec_in_rxns = set([rlt.id_in for rlt in padmetSpec.dicOfRelationOut.get(pwy_id,[]) if rlt.type == "is_in_pathway"])

                if only_complete_pathways:
                    difference_rxns = padmet_ref_in_rxns.difference(padmet_spec_in_rxns)

                    if difference_rxns != set():
                        if difference_rxns.issubset(all_spontaneous_rxns):
                            for difference_rxn in difference_rxns:
                                if difference_rxn not in set([node.id for node in list(padmetSpec.dicOfNode.values()) if node.type == "reaction"]):
                                    padmetSpec.copyNode(padmetRef, difference_rxn)
                                    number_spontaneous_reactions += 1
                else:
                    if spontaneous_rxn_id not in set([node.id for node in list(padmetSpec.dicOfNode.values()) if node.type == "reaction"]):
                        padmetSpec.copyNode(padmetRef, spontaneous_rxn_id)
                        number_spontaneous_reactions += 1

    padmetSpec.generateFile(output_padmet_path)

    print('Add {0} spontaneous reactions to {1}'.format(number_spontaneous_reactions, output_padmet_path))


def create_output(tmp_study_data):
    padmet_path = tmp_study_data['padmet_path']
    study_padmet = tmp_study_data['study_padmet'].replace('.padmet', '').replace('output_pathwaytools_', '')
    verbose = tmp_study_data['verbose']
    veryverbose = tmp_study_data['veryverbose']
    padmet_from_networks_path = tmp_study_data['padmet_from_networks_path']
    sbml_from_networks_path = tmp_study_data['sbml_from_networks_path']
    padmet_ref_path = tmp_study_data['database_path']

    if not os.path.exists(padmet_from_networks_path + '/' + study_padmet + '.padmet'):
        if verbose:
            print('Create ' + study_padmet +' from ' + padmet_path + ' to ' + padmet_from_networks_path)
        add_spontaneous_reactions(padmet_path, padmet_ref_path, padmet_from_networks_path + '/' + study_padmet + '.padmet')
    else:
        print('There is already a padmet for ' + study_padmet + ' ' + padmet_from_networks_path + '.')

    if not os.path.exists(sbml_from_networks_path + '/' + study_padmet + '.sbml'):
        sbmlGenerator.padmet_to_sbml(padmet=padmet_path, output=sbml_from_networks_path + '/' + study_padmet + '.sbml', verbose=veryverbose)
    else:
        print('There is already a sbml for ' + study_padmet + ' in ' + sbml_from_networks_path + '.')
